@@ -186,6 +186,19 @@ class SyncOwnAccountTask(Task):
                                    remote.get('username'),
                                    remote.get('email'))
 
+class GetVersionTask(Task):
+    def __repr__(self):
+        return '<GetVersionTask>'
+
+    def __eq__(self, other):
+        if other.__class__ == self.__class__:
+            return True
+        return False
+
+    def run(self, sync):
+        version = sync.get('config/server/version')
+        sync.setRemoteVersion(version)
+
 class SyncProjectListTask(Task):
     def __repr__(self):
         return '<SyncProjectListTask>'
@@ -567,7 +580,7 @@ class SyncChangeTask(Task):
             change.subject = remote_change['subject']
             change.updated = dateutil.parser.parse(remote_change['updated'])
             change.topic = remote_change.get('topic')
-            repo = app.getRepo(change.project.name)
+            repo = gitrepo.get_repo(change.project.name, app.config)
             new_revision = False
             for remote_commit, remote_revision in remote_change.get('revisions', {}).items():
                 revision = session.getRevisionByCommit(remote_commit)
@@ -653,6 +666,8 @@ class SyncChangeTask(Task):
                             if remote_comment.get('side', '') == 'PARENT':
                                 parent = True
                             fileobj = revision.getFile(remote_file)
+                            if fileobj is None:
+                                fileobj = revision.createFile(remote_file, 'M')
                             comment = fileobj.createComment(remote_comment['id'], account,
                                                             remote_comment.get('in_reply_to'),
                                                             created,
@@ -847,7 +862,7 @@ class CheckReposTask(Task):
             try:
                 missing = False
                 try:
-                    app.getRepo(project.name)
+                    repo = gitrepo.get_repo(project.name, app.config)
                 except gitrepo.GitCloneError:
                     missing = True
                 if missing or app.fetch_missing_refs:
@@ -878,7 +893,7 @@ class CheckRevisionsTask(Task):
             project = session.getProject(self.project_key)
             repo = None
             try:
-                repo = app.getRepo(project.name)
+                repo = gitrepo.get_repo(project.name, app.config)
             except gitrepo.GitCloneError:
                 pass
             for change in project.open_changes:
@@ -1078,9 +1093,17 @@ class ChangeCommitMessageTask(Task):
             revision.pending_message = False
             data = dict(message=revision.message)
             # Inside db session for rollback
-            sync.post('changes/%s/revisions/%s/message' %
-                      (revision.change.id, revision.commit),
-                      data)
+            if sync.version < (2,11,0):
+                sync.post('changes/%s/revisions/%s/message' %
+                          (revision.change.id, revision.commit),
+                          data)
+            else:
+                edit = sync.get('changes/%s/edit' % revision.change.id)
+                if edit is not None:
+                    raise Exception("Edit already in progress on change %s" %
+                                    (revision.change.number,))
+                sync.put('changes/%s/edit:message' % (revision.change.id,), data)
+                sync.post('changes/%s/edit:publish' % (revision.change.id,), {})
             change_id = revision.change.id
         sync.submitTask(SyncChangeTask(change_id, priority=self.priority))
 
@@ -1209,7 +1232,7 @@ class PruneChangeTask(Task):
             change = session.getChange(self.key)
             if not change:
                 return
-            repo = app.getRepo(change.project.name)
+            repo = gitrepo.get_repo(change.project.name, app.config)
             self.log.info("Pruning %s change %s status:%s updated:%s" % (
                 change.project.name, change.number, change.status, change.updated))
             change_ref = None
@@ -1249,6 +1272,7 @@ class Sync(object):
     def __init__(self, app):
         self.user_agent = 'Gertty/%s %s' % (gertty.version.version_info.release_string(),
                                             requests.utils.default_user_agent())
+        self.version = (0, 0, 0)
         self.offline = False
         self.account_id = None
         self.app = app
@@ -1262,6 +1286,7 @@ class Sync(object):
             authclass = requests.auth.HTTPDigestAuth
         self.auth = authclass(
             self.app.config.username, self.app.config.password)
+        self.submitTask(GetVersionTask(HIGH_PRIORITY))
         self.submitTask(SyncOwnAccountTask(HIGH_PRIORITY))
         self.submitTask(CheckReposTask(HIGH_PRIORITY))
         self.submitTask(UploadReviewsTask(HIGH_PRIORITY))
@@ -1308,6 +1333,7 @@ class Sync(object):
         except requests.ConnectionError, e:
             self.log.warning("Offline due to: %s" % (e,))
             if not self.offline:
+                self.submitTask(GetVersionTask(HIGH_PRIORITY))
                 self.submitTask(UploadReviewsTask(HIGH_PRIORITY))
             self.offline = True
             self.app.status.update(offline=True, refresh=False)
@@ -1413,3 +1439,16 @@ class Sync(object):
                 return
         task = SyncChangesByCommitsTask([commit], priority)
         self.submitTask(task)
+
+    def setRemoteVersion(self, version):
+        base = version.split('-')[0]
+        parts = base.split('.')
+        major = minor = micro = 0
+        if len(parts) > 0:
+            major = int(parts[0])
+        if len(parts) > 1:
+            minor = int(parts[1])
+        if len(parts) > 2:
+            micro = int(parts[2])
+        self.version = (major, minor, micro)
+        self.log.info("Remote version is: %s (parsed as %s)" % (version, self.version))
